﻿Imports GoogleAnalytics
Imports System.Text
Imports System.Threading.Tasks
Imports Telerik.WinControls.Analytics

Namespace ERP.Client.Analytics
	Public Class GoogleAnalyticsMonitor
		Implements ITraceMonitor
'INSTANT VB NOTE: The variable tracker was renamed since Visual Basic does not handle variables named the same as their type well:
		Private tracker_Renamed As Tracker
		Private featureTimings As IDictionary(Of String, Date)

		Public Sub New()
			Me.CreateGoogleTracker()
			Me.featureTimings = New Dictionary(Of String, Date)()
		End Sub

		Public Sub CreateGoogleTracker()
			Dim trackerManager_Renamed = New TrackerManager(New PlatformInfoProvider())
			tracker_Renamed = trackerManager_Renamed.CreateTracker("UA-111455-55")
			tracker_Renamed.AppName = "WinForms ERP"
		End Sub

		Public Sub TrackAtomicFeature(ByVal feature As String) Implements ITraceMonitor.TrackAtomicFeature
			Dim category As String
			Dim eventAction As String
			Me.SplitFeatureName(feature, category, eventAction)
			Dim data = HitBuilder.CreateCustomEvent(category, eventAction & " event", feature.ToString(), 1).Build()
			tracker_Renamed.Send(data)
		End Sub

		Public Sub TrackError(ByVal feature As String, ByVal exception_Renamed As Exception) Implements ITraceMonitor.TrackError
			Dim data = HitBuilder.CreateException(feature & ":" & exception_Renamed.ToString(), True).Build()
			tracker_Renamed.Send(data)
		End Sub
		Public Sub TrackFeatureCancel(ByVal feature As String) Implements ITraceMonitor.TrackFeatureCancel
			Dim category As String
			Dim eventAction As String
			Me.SplitFeatureName(feature, category, eventAction)
			If Me.featureTimings.ContainsKey(eventAction) Then
				Me.featureTimings.Remove(eventAction)
			End If

			Dim data = HitBuilder.CreateCustomEvent(category, eventAction & " event.Cancelled", feature.ToString(), 1).Build()
			tracker_Renamed.Send(data)
		End Sub


		Public Sub TrackFeatureStart(ByVal feature As String) Implements ITraceMonitor.TrackFeatureStart
			' Measuring timings provides a native way to measure a period of time in Google Analytics. 
			' This can be useful to measure resource load times, for example.
			'TimeSpan ts = TimeSpan.FromSeconds(2.2);
			Dim category As String
			Dim eventAction As String
			Me.SplitFeatureName(feature, category, eventAction)
			If Not Me.featureTimings.ContainsKey(eventAction) Then
				Me.featureTimings.Add(eventAction, Date.Now)
			Else
				Me.featureTimings(eventAction) = Date.Now
			End If

			Dim loadTiming = HitBuilder.CreateTiming(category, eventAction, TimeSpan.FromSeconds(0)).Build()
			tracker_Renamed.Send(loadTiming)
		End Sub

		Public Sub TrackFeatureEnd(ByVal feature As String) Implements ITraceMonitor.TrackFeatureEnd
			Dim category As String
			Dim eventAction As String
			Me.SplitFeatureName(feature, category, eventAction)
			Dim timing? As TimeSpan = Nothing
			If Me.featureTimings.ContainsKey(eventAction) Then
				timing = Date.Now.Subtract(Me.featureTimings(eventAction))
				Me.featureTimings.Remove(eventAction)
			End If

			Dim unLoadTiming = HitBuilder.CreateTiming(category, eventAction, timing).Build()
			tracker_Renamed.Send(unLoadTiming)
		End Sub

		Public Sub TrackValue(ByVal feature As String, ByVal value As Long) Implements ITraceMonitor.TrackValue
			Dim category As String
			Dim eventAction As String
			Me.SplitFeatureName(feature, category, eventAction)
			Dim data = HitBuilder.CreateCustomEvent(category, eventAction & " event", feature.ToString(), value).Build()
			tracker_Renamed.Send(data)
		End Sub

		Private Sub SplitFeatureName(ByVal feature As String, <System.Runtime.InteropServices.Out()> ByRef category As String, <System.Runtime.InteropServices.Out()> ByRef eventAction As String)
			Dim parameters() As String = feature.Split("."c)
			category = parameters(0)
			eventAction = parameters(1)
		End Sub
	End Class
End Namespace
