﻿Imports System.ComponentModel
Imports System.Text
Imports System.Threading.Tasks
Imports Telerik.WinControls.UI
Imports Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx
Imports System.IO
Imports Telerik.Windows.Documents.Spreadsheet.Model
Imports Telerik.WinForms.Controls.Spreadsheet
Imports Telerik.Windows.Documents.Spreadsheet.FormatProviders
Imports ERP.Repository
Imports Telerik.WinControls

Namespace ERP.Client
	Partial Public Class BaseGridControl
		Inherits UserControl
		Protected columnNames As List(Of String)
		Protected currentItem As ISavableObject
		Protected columnTypes() As Type
		Protected dataFormText As String
		Public ReadOnly Property gridControl() As RadVirtualGrid
			Get
				Return Me.radGridView1
			End Get
		End Property

		Public Sub New()
			InitializeComponent()
			columnNames = New List(Of String)()
			Me.radGridView1.AllowFiltering = True
			Me.radGridView1.AllowAddNewRow = False
			Me.radGridView1.AutoSizeColumnsMode = VirtualGridAutoSizeColumnsMode.Fill
			AddHandler Me.radGridView1.CellValueNeeded, AddressOf RadGridView1_CellValueNeeded
			Me.radGridView1.EnablePaging = True
			Me.radGridView1.PageSize = 20
			AddHandler Me.radGridView1.VirtualGridElement.PageIndexChanged, AddressOf VirtualGridElement_PageIndexChanged
			AddHandler Me.radGridView1.VirtualGridElement.PageIndexChanging, AddressOf VirtualGridElement_PageIndexChanging
			AddHandler Me.radGridView1.CellEditorInitialized, AddressOf RadGridView1_CellEditorInitialized
			Me.radGridView1.BeginEditMode = RadVirtualGridBeginEditMode.BeginEditProgrammatically
			Me.radGridView1.VirtualGridElement.TableElement.PagingPanelElement.Margin = New Padding(0, 0, 0, 1)
			Me.radGridView1.AllowMultiColumnSorting = False
			AddHandler Me.radGridView1.SortChanged, AddressOf RadGridView1_SortChanged
			Me.Initialize()
		End Sub

		Protected Overridable Sub RadGridView1_SortChanged(ByVal sender As Object, ByVal e As VirtualGridEventArgs)
			Dim skip = e.ViewInfo.PageSize * radGridView1.PageIndex
			RefreshData(skip)
		End Sub

		Private Sub RadGridView1_CellEditorInitialized(ByVal sender As Object, ByVal e As VirtualGridCellEditorInitializedEventArgs)
			If e.RowIndex = RadVirtualGrid.FilterRowIndex Then
				Dim editor = TryCast(e.ActiveEditor, BaseInputEditor)
				Dim element = TryCast(editor.EditorElement, RadItem)
				If TypeOf element Is RadSpinEditorElement Then
					AddHandler (CType(element, RadSpinEditorElement)).TextBoxItem.KeyDown, AddressOf Element_KeyDown
				ElseIf TypeOf element Is RadDateTimeEditorElement Then
					AddHandler (CType(element, RadDateTimeEditorElement)).KeyDown, AddressOf Element_KeyDown
				ElseIf TypeOf element Is RadTextBoxEditorElement Then
					AddHandler (CType(element, RadTextBoxEditorElement)).TextBoxItem.KeyDown, AddressOf Element_KeyDown
				End If
			End If
		End Sub

        Private Sub Element_KeyDown(ByVal sender As Object, ByVal e As Forms.KeyEventArgs)
            If e.KeyCode = Keys.Enter Then
                RefreshData(0)
                Me.gridControl.PageIndex = 0
            End If
        End Sub

        Protected async Sub ExecuteQueryAsync(Of T)(ByVal task_Renamed As Task(Of T), ByVal callback As Action(Of T))
			Dim result = await task_Renamed
			callback(result)
		End Sub

		Protected Overridable Sub VirtualGridElement_PageIndexChanging(ByVal sender As Object, ByVal e As VirtualGridPageChangingEventArgs)
			Dim skip = e.ViewInfo.PageSize * e.NewIndex
			RefreshData(skip)
		End Sub

		Protected Overridable Sub Initialize()

		End Sub

		Protected Overridable Sub VirtualGridElement_PageIndexChanged(ByVal sender As Object, ByVal e As VirtualGridEventArgs)

		End Sub

		Protected Overridable Sub RadGridView1_CellValueNeeded(ByVal sender As Object, ByVal e As VirtualGridCellValueNeededEventArgs)

		End Sub

		Protected Overridable Sub EditButton_Click(ByVal sender As Object, ByVal e As EventArgs) Handles editButton.Click
			If Me.gridControl.CurrentCell Is Nothing Then
				RadMessageBox.Show("Please select a row.")
				Return
			End If

			Dim dialog = New ERPDataForm()
			dialog.Text = dataFormText
			dialog.DataEntry.DataSource = currentItem

			If dialog.ShowDialog() = DialogResult.OK Then
				currentItem.Save(False)
				Me.RefreshData(Me.gridControl.PageSize * Me.gridControl.PageIndex)

			End If
		End Sub

		Protected Overridable Sub DeleteButton_Click(ByVal sender As Object, ByVal e As EventArgs) Handles deleteButton.Click
			If Me.gridControl.CurrentCell Is Nothing Then
				RadMessageBox.Show("Please select a row!")
				Return
			End If

			If RadMessageBox.Show("Delete Selected Item?", "Confirm Delete", MessageBoxButtons.YesNo) = DialogResult.Yes Then
				TryCast(currentItem, ISavableObject).Delete()
				Me.RefreshData(Me.gridControl.PageSize * Me.gridControl.PageIndex)
			End If
		End Sub

		Protected Overridable Sub PrintButton_Click(ByVal sender As Object, ByVal e As EventArgs) Handles printButton.Click
			If Me.gridControl.MasterViewInfo.IsWaiting Then
				RadMessageBox.Show("The Data is not loaded! Please wait!")
				Return
			End If

			Dim spreadsheet = New RadSpreadsheet()
			Dim window = New Form() With {.Width = 0, .Height = 0, .Opacity = 0}
			spreadsheet.Parent = window
			window.Show()
			spreadsheet.Workbook = CreateWorkbook()
			spreadsheet.Print(New PrintWhatSettings(ExportWhat.ActiveSheet, False))
			window.Close()
		End Sub

		Protected Overridable Sub ExportButton_Click(ByVal sender As Object, ByVal e As EventArgs) Handles exportButton.Click
			If Me.gridControl.MasterViewInfo.IsWaiting Then
				RadMessageBox.Show("The Data is not loaded! Please wait!")
				Return
			End If

			Dim savefile As New SaveFileDialog()
			savefile.FileName = "Document.xlsx"
			savefile.Filter = "Excel files (*.xlsx)|*.xlsx|All files (*.*)|*.*"

			If savefile.ShowDialog() = DialogResult.OK Then
				Dim formatProvider = New XlsxFormatProvider()

				Using output As Stream = New FileStream(savefile.FileName, FileMode.Create)
					formatProvider.Export(CreateWorkbook(), output)
				End Using
			End If
		End Sub

		Protected Overridable Function CreateWorkbook() As Workbook
			Return New Workbook()
		End Function
		Protected Overridable Sub RefreshData(ByVal skip As Integer)

		End Sub

	End Class
End Namespace
