﻿Imports System.ComponentModel
Imports System.Text
Imports System.Threading.Tasks
Imports Telerik.WinControls.UI
Imports ERP.Repository.Service
Imports ERP.Repository

Namespace ERP.Client
	Partial Public Class ERPDataDialog
		Inherits UserControl
		Public Event CommandExecuted As EventHandler(Of DataDialogCommandEventArgs)
		Public Sub New()
			InitializeComponent()
            Me.radDataEntry.ItemDefaultSize = New System.Drawing.Size(300, 25)
            Me.radDataEntry.FlowDirection = Forms.FlowDirection.TopDown
            Me.radDataEntry.DataEntryElement.ItemSpace = 10
			Me.radDataEntry.FitToParentWidth = True

			AddHandler Me.radDataEntry.ItemInitializing, AddressOf radDataEntry_ItemInitializing
			AddHandler Me.radDataEntry.BindingCreating, AddressOf radDataEntry_BindingCreating
			AddHandler Me.radDataEntry.BindingCreated, AddressOf radDataEntry_BindingCreated
			AddHandler Me.radDataEntry.EditorInitializing, AddressOf radDataEntry_EditorInitializing
			AddHandler Me.radDataEntry.VerticalScrollbar.ValueChanged, AddressOf VerticalScrollbar_ValueChanged
		End Sub

		Private Sub VerticalScrollbar_ValueChanged(ByVal sender As Object, ByVal e As EventArgs)
			Me.radDataEntry.PanelContainer.Invalidate()
		End Sub

		Protected Overridable Sub OnCommandExecuted(ByVal e As DataDialogCommandEventArgs)
			RaiseEvent CommandExecuted(Me, e)
		End Sub

		Private Sub radDataEntry_ItemInitializing(ByVal sender As Object, ByVal e As ItemInitializingEventArgs)

			If TypeOf radDataEntry.DataSource Is Address Then
				If Not FieldsHelper.AddressFields.Contains(e.Panel.Controls(1).Text) Then
					e.Cancel = True
					e.Panel.Visible = False

				End If
			ElseIf TypeOf radDataEntry.DataSource Is PurchaseOrderHeader Then
				If Not FieldsHelper.PurchaseOrderHeaderFields.Contains(e.Panel.Controls(1).Text) Then
					e.Cancel = True
					e.Panel.Visible = False
				End If
			ElseIf TypeOf radDataEntry.DataSource Is Vendor Then
				If Not FieldsHelper.VendorFields.Contains(e.Panel.Controls(1).Text) Then
					e.Cancel = True
					e.Panel.Visible = False
				End If
			ElseIf TypeOf radDataEntry.DataSource Is ProductInventory Then
				If Not FieldsHelper.InventoriesFields.Contains(e.Panel.Controls(1).Text) Then
					e.Cancel = True
					e.Panel.Visible = False
				End If
			ElseIf TypeOf radDataEntry.DataSource Is WorkOrder Then
				If Not FieldsHelper.WorkOrdersFields.Contains(e.Panel.Controls(1).Text) Then
					e.Cancel = True
					e.Panel.Visible = False
				End If
			ElseIf TypeOf radDataEntry.DataSource Is BillOfMaterial Then
				If Not FieldsHelper.BillOfMaterialsFields.Contains(e.Panel.Controls(1).Text) Then
					e.Cancel = True
					e.Panel.Visible = False
				End If
			ElseIf TypeOf radDataEntry.DataSource Is SalesOrderHeader Then
				If Not FieldsHelper.OrdersFields.Contains(e.Panel.Controls(1).Text) Then
					e.Cancel = True
					e.Panel.Visible = False
				End If
			ElseIf TypeOf radDataEntry.DataSource Is Customer Then
				Dim customer = TryCast(radDataEntry.DataSource, Customer)
				If customer.IsPerson Then
					If Not FieldsHelper.IndividualsFields.Contains(e.Panel.Controls(1).Text) Then
						e.Cancel = True
						e.Panel.Visible = False
					End If
				Else
					If Not FieldsHelper.StoresFields.Contains(e.Panel.Controls(1).Text) Then
						e.Cancel = True
						e.Panel.Visible = False
					End If
				End If

			End If

		End Sub



		Private Sub radDataEntry_EditorInitializing(ByVal sender As Object, ByVal e As EditorInitializingEventArgs)
			If TypeOf radDataEntry.DataSource Is Address Then
				If e.Property.Name = "StateProvince" Then
					Dim radDropDownList1 = New RadDropDownList()
					radDropDownList1.DataSource = MainRepository.StatesCache
					radDropDownList1.ValueMember = "StateProvinceID"
					radDropDownList1.DisplayMember = "FullName"
					radDropDownList1.DropDownStyle = Telerik.WinControls.RadDropDownStyle.DropDownList
					e.Editor = radDropDownList1
				End If

			ElseIf TypeOf radDataEntry.DataSource Is PurchaseOrderHeader Then
				Dim purchaseOrder = TryCast(Me.radDataEntry.DataSource, PurchaseOrderHeader)

				If e.Property.Name = "OrderStatus" Then
					Dim radDropDownList1 = New RadDropDownList()
					radDropDownList1.DataSource = purchaseOrder.OrderStatuses
					radDropDownList1.DropDownStyle = Telerik.WinControls.RadDropDownStyle.DropDownList
					e.Editor = radDropDownList1
				ElseIf e.Property.Name = "ShipMethod" Then
					Dim radDropDownList1 = New RadDropDownList()
					radDropDownList1.DropDownStyle = Telerik.WinControls.RadDropDownStyle.DropDownList
					radDropDownList1.DataSource = MainRepository.ShipMethodsCache
					radDropDownList1.ValueMember = "ShipMethodID"
					radDropDownList1.DisplayMember = "Name"
					e.Editor = radDropDownList1
				ElseIf e.Property.Name = "Vendor" Then
					Dim radDropDownList1 = New RadDropDownList()
					radDropDownList1.DropDownStyle = Telerik.WinControls.RadDropDownStyle.DropDownList
					radDropDownList1.DataSource = MainRepository.VendorsCache
					radDropDownList1.ValueMember = "BusinessEntityID"
					radDropDownList1.DisplayMember = "Name"
					e.Editor = radDropDownList1
				End If

			ElseIf TypeOf radDataEntry.DataSource Is Customer Then

				If e.Property.Name = "State" Then
					Dim radDropDownList1 = New RadDropDownList()
					radDropDownList1.DropDownStyle = Telerik.WinControls.RadDropDownStyle.DropDownList
					radDropDownList1.DataSource = MainRepository.StatesCache
					radDropDownList1.ValueMember = "StateProvinceID"
					radDropDownList1.DisplayMember = "FullName"
					e.Editor = radDropDownList1
				End If
			ElseIf TypeOf radDataEntry.DataSource Is BillOfMaterial Then

				If e.Property.Name = "UnitMeasure" Then
					Dim radDropDownList1 = New RadDropDownList()
					radDropDownList1.DropDownStyle = Telerik.WinControls.RadDropDownStyle.DropDownList
					radDropDownList1.DataSource = MainRepository.UnitMeasuresCache
					radDropDownList1.ValueMember = "UnitMeasureCode"
					radDropDownList1.DisplayMember = "Name"
					e.Editor = radDropDownList1
				ElseIf e.Property.Name = "Product1" Then
					Dim radDropDownList1 = New RadDropDownList()
					radDropDownList1.DropDownStyle = Telerik.WinControls.RadDropDownStyle.DropDownList
					radDropDownList1.DataSource = MainRepository.ProductsCache.ToList()
					radDropDownList1.ValueMember = "ProductID"
					radDropDownList1.DisplayMember = "Name"
					e.Editor = radDropDownList1
				ElseIf e.Property.Name = "Product" Then
					Dim radDropDownList1 = New RadDropDownList()
					radDropDownList1.DropDownStyle = Telerik.WinControls.RadDropDownStyle.DropDownList
					radDropDownList1.DataSource = MainRepository.ProductsCache.ToList()
					radDropDownList1.ValueMember = "ProductID"
					radDropDownList1.DisplayMember = "Name"

					e.Editor = radDropDownList1
				End If
			ElseIf TypeOf radDataEntry.DataSource Is ProductInventory Then
				If e.Property.Name = "Product" Then
					Dim radDropDownList1 = New RadDropDownList()
					radDropDownList1.DropDownStyle = Telerik.WinControls.RadDropDownStyle.DropDownList
					radDropDownList1.DataSource = MainRepository.ProductsCache
					radDropDownList1.ValueMember = "ProductID"
					radDropDownList1.DisplayMember = "Name"
					e.Editor = radDropDownList1
				ElseIf e.Property.Name = "Color" Then
					e.Editor = New RadColorBox()
				ElseIf e.Property.Name = "Location" Then
					Dim radDropDownList1 = New RadDropDownList()
					radDropDownList1.DropDownStyle = Telerik.WinControls.RadDropDownStyle.DropDownList
					radDropDownList1.DataSource = MainRepository.LocationsCache
					radDropDownList1.ValueMember = "LocationID"
					radDropDownList1.DisplayMember = "Name"
					e.Editor = radDropDownList1
				End If
			ElseIf TypeOf radDataEntry.DataSource Is SalesOrderHeader Then
				Dim order = TryCast(radDataEntry.DataSource, SalesOrderHeader)

				If e.Property.Name = "Customer" Then
					Dim customers = MainRepository.TopCustomersCache
					Dim orderCustomer = order.Customer
					If orderCustomer IsNot Nothing AndAlso (Not customers.Contains(orderCustomer)) Then
						customers.Insert(0, orderCustomer)
					End If

					Dim radDropDownList1 = New RadDropDownList()
					radDropDownList1.DropDownStyle = Telerik.WinControls.RadDropDownStyle.DropDownList
					radDropDownList1.DataSource = customers
					radDropDownList1.ValueMember = "CustomerID"
					radDropDownList1.DisplayMember = "Name"
					e.Editor = radDropDownList1
				ElseIf e.Property.Name = "ShipMethod" Then
					Dim radDropDownList1 = New RadDropDownList()
					radDropDownList1.DataSource = MainRepository.ShipMethodsCache
					radDropDownList1.DropDownStyle = Telerik.WinControls.RadDropDownStyle.DropDownList
					radDropDownList1.ValueMember = "ShipMethodID"
					radDropDownList1.DisplayMember = "Name"
					e.Editor = radDropDownList1
				End If
			ElseIf TypeOf radDataEntry.DataSource Is Vendor Then
				If e.Property.Name = "CreditRating" Then
					Dim editor = New RadSpinEditor()
					editor.Maximum = 5
					editor.Minimum = 0
					e.Editor = editor
				End If
			ElseIf TypeOf radDataEntry.DataSource Is WorkOrder Then
				If e.Property.Name = "Product" Then
					Dim radDropDownList1 = New RadDropDownList()
					radDropDownList1.DropDownStyle = Telerik.WinControls.RadDropDownStyle.DropDownList
					radDropDownList1.DataSource = MainRepository.ProductsCache
					radDropDownList1.ValueMember = "ProductID"
					radDropDownList1.DisplayMember = "Name"
					e.Editor = radDropDownList1
				End If
			End If
		End Sub

		Private Sub radDataEntry_BindingCreating(ByVal sender As Object, ByVal e As BindingCreatingEventArgs)
			If TypeOf radDataEntry.DataSource Is Address Then
				If e.DataMember = "StateProvince" Then
					e.DataMember = "StateProvince.StateProvinceID"
					e.PropertyName = "SelectedValue"
				End If
			ElseIf TypeOf radDataEntry.DataSource Is PurchaseOrderHeader Then
				If e.DataMember = "ShipMethod" OrElse e.DataMember = "OrderStatus" OrElse e.DataMember = "Vendor" Then
					e.PropertyName = "SelectedValue"
				End If
			ElseIf TypeOf radDataEntry.DataSource Is Customer Then
				If e.DataMember = "State" Then
					e.DataMember = "Address.StateProvinceID"
					e.PropertyName = "SelectedValue"
				End If
			ElseIf TypeOf radDataEntry.DataSource Is BillOfMaterial Then
				Dim entry = TryCast(radDataEntry.DataSource, BillOfMaterial)

				If e.DataMember = "Product" Then
					e.DataMember = "ComponentID"
					e.PropertyName = "SelectedValue"
				End If
				If e.DataMember = "Product1" Then
					e.DataMember = "ProductAssemblyID"
					e.PropertyName = "SelectedValue"
				ElseIf e.DataMember = "UnitMeasure" Then
					e.DataMember = "UnitMeasure.UnitMeasureCode"
					e.PropertyName = "SelectedValue"
				End If

			End If

		End Sub

		Private Sub radDataEntry_BindingCreated(ByVal sender As Object, ByVal e As BindingCreatedEventArgs)
			If TypeOf radDataEntry.DataSource Is ProductInventory Then
				If e.DataMember = "Color" Then
					e.Binding.FormattingEnabled = True
					AddHandler e.Binding.Format, AddressOf Binding_Format
				End If
			End If
		End Sub

		Private Sub Binding_Format(ByVal sender As Object, ByVal e As ConvertEventArgs)
            If e.DesiredType Is GetType(System.Drawing.Color) AndAlso e.Value IsNot Nothing Then
                Dim color = System.Drawing.Color.Transparent
                If e.Value.ToString().StartsWith("#") Then
                    color = ColorTranslator.FromHtml(e.Value.ToString())
                ElseIf Not e.Value.ToString().Contains("/"c) Then
                    color = System.Drawing.Color.FromName(e.Value.ToString())
                End If
                e.Value = color
            End If
        End Sub

		Private Sub okButton_Click(ByVal sender As Object, ByVal e As EventArgs) Handles okButton.Click
			Me.OnCommandExecuted(New DataDialogCommandEventArgs(True))

			Dim form = TryCast(Me.Parent, ERPDataForm)
			If form IsNot Nothing Then
				form.DialogResult = DialogResult.OK
				form.Close()
			End If

		End Sub

		Private Sub cancelButton_Click(ByVal sender As Object, ByVal e As EventArgs) Handles cancelButton.Click
			Me.OnCommandExecuted(New DataDialogCommandEventArgs(False))

			Dim form = TryCast(Me.Parent, ERPDataForm)
			If form IsNot Nothing Then
				form.DialogResult = DialogResult.Cancel
				form.Close()
			End If

		End Sub
		Public ReadOnly Property DataEntry() As RadDataEntry
			Get
				Return Me.radDataEntry
			End Get
		End Property
	End Class
End Namespace
