﻿Imports ERP.Repository
Imports ERP.Repository.Service
Imports System.ComponentModel
Imports System.Data.Services.Client
Imports System.Text
Imports System.Threading.Tasks
Imports Telerik.WinControls.UI
Imports System.Data.Entity
Imports Telerik.WinControls.Export
Imports Telerik.Windows.Documents.Spreadsheet.Model
Imports Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx
Imports System.IO
Imports Telerik.WinControls

Namespace ERP.Client
	Friend Class IndividualsControl
		Inherits BaseGridControl

        Private relatedOrdersControl1 As RelatedOrdersControl

        Private data As List(Of Customer)
        Private privateCallback As Action(Of List(Of Customer))
		Public Property Callback() As Action(Of List(Of Customer))
			Get
				Return privateCallback
			End Get
			Private Set(ByVal value As Action(Of List(Of Customer)))
				privateCallback = value
			End Set
		End Property

        Protected Overrides Sub Initialize()
            data = New List(Of Customer)()
            Me.dataFormText = "Edit Individual"
            relatedOrdersControl1 = New RelatedOrdersControl()
            relatedOrdersControl1.Parent = Me
            relatedOrdersControl1.Dock = DockStyle.Bottom
            relatedOrdersControl1.Height = 300

            AddHandler Me.gridControl.SelectionChanged, AddressOf GridControl_SelectionChanged
            Me.gridControl.ColumnCount = 9
            columnTypes = New Type() {GetType(String), GetType(String), GetType(String), GetType(String), GetType(String), GetType(String), GetType(String), GetType(String), GetType(Date)}
            Me.columnNames.Add("Account Number")
            Me.columnNames.Add("First Name")
            Me.columnNames.Add("Last Name")
            Me.columnNames.Add("Email Address")
            Me.columnNames.Add("Phone")
            Me.columnNames.Add("Address Line 1")
            Me.columnNames.Add("City")
            Me.columnNames.Add("State")
            Me.columnNames.Add("Modified Date")

            Me.gridControl.RowCount = MainRepository.GetIndividualCustomers().Count()

            Me.Callback = New Action(Of List(Of Customer))(Sub(query)
                                                               Me.data = query
                                                               Me.gridControl.MasterViewInfo.IsWaiting = False
                                                               Me.gridControl.TableElement.SynchronizeRows()
                                                           End Sub)

            Me.RefreshData(0)
            Me.gridControl.MasterViewInfo.SetColumnDataType(columnTypes)
            AddHandler Me.gridControl.CellFormatting, AddressOf GridControl_CellFormatting
        End Sub

        Private Sub GridControl_CellFormatting(ByVal sender As Object, ByVal e As VirtualGridCellElementEventArgs)
			If TypeOf e.CellElement Is VirtualGridFilterCellElement Then
				If e.CellElement.ColumnIndex = 1 OrElse e.CellElement.ColumnIndex = 2 OrElse e.CellElement.ColumnIndex = 3 OrElse e.CellElement.ColumnIndex = 4 OrElse e.CellElement.ColumnIndex = 5 OrElse e.CellElement.ColumnIndex = 6 OrElse e.CellElement.ColumnIndex = 7 Then
					e.CellElement.Enabled = False
				Else
					e.CellElement.ResetValue(RadItem.EnabledProperty, ValueResetFlags.Local)
				End If
			End If
			If TypeOf e.CellElement Is VirtualGridHeaderCellElement Then
				Dim cell = TryCast(e.CellElement, VirtualGridHeaderCellElement)
				If e.CellElement.ColumnIndex = 1 OrElse e.CellElement.ColumnIndex = 2 OrElse e.CellElement.ColumnIndex = 3 OrElse e.CellElement.ColumnIndex = 4 OrElse e.CellElement.ColumnIndex = 5 OrElse e.CellElement.ColumnIndex = 6 OrElse e.CellElement.ColumnIndex = 7 Then
					cell.Arrow.ShouldPaint = False
				Else
					cell.Arrow.ResetValue(LightVisualElement.ShouldPaintProperty, ValueResetFlags.Local)
				End If
			End If
		End Sub

		Private Sub GridControl_SelectionChanged(ByVal sender As Object, ByVal e As EventArgs)
			If Me.gridControl.CurrentCell IsNot Nothing AndAlso Me.gridControl.CurrentCell.RowIndex >= 0 Then
				Dim item = data(Me.gridControl.CurrentCell.RowIndex Mod gridControl.PageSize)
                relatedOrdersControl1.OrdersGrid.DataSource = item.SalesOrderHeaders
                currentItem = data(Me.gridControl.CurrentCell.RowIndex Mod Me.gridControl.PageSize)
			End If

		End Sub

		Protected Overrides Sub RadGridView1_CellValueNeeded(ByVal sender As Object, ByVal e As VirtualGridCellValueNeededEventArgs)
			MyBase.RadGridView1_CellValueNeeded(sender, e)
			If e.ColumnIndex < 0 Then
				Return
			End If
			If e.RowIndex < 0 Then
				e.FieldName = FieldsHelper.IndividualsFields(e.ColumnIndex)

			End If
			If e.RowIndex = RadVirtualGrid.HeaderRowIndex Then
				e.Value = columnNames(e.ColumnIndex)
			End If
			If e.RowIndex >= 0 AndAlso data.Count > 0 Then
				Dim rowData = TryCast(data(e.RowIndex Mod gridControl.PageSize), Customer)

				Select Case e.ColumnIndex
					Case 0
						e.Value = rowData.AccountNumber
					Case 1
						e.Value = rowData.FirstName
					Case 2
						e.Value = rowData.LastName
					Case 3
						e.Value = rowData.EmailAddress
					Case 4
						e.Value = rowData.Phone
					Case 5
						e.Value = rowData.AddressLine1
					Case 6
						e.Value = rowData.City
					Case 7
						e.Value = rowData.State
					Case 8
						e.Value = rowData.ModifiedDate
						e.FormatString = "{0:MM/dd/yyyy}"
				End Select
			End If
		End Sub
		Protected Overrides Sub RadGridView1_SortChanged(ByVal sender As Object, ByVal e As VirtualGridEventArgs)
			If e.ViewInfo.SortDescriptors.Count = 0 Then
				Return
			End If

			Dim propertyName = e.ViewInfo.SortDescriptors(0).PropertyName
			If propertyName Is "FirstName" OrElse propertyName Is "LastName" OrElse propertyName Is "EmailAddress" OrElse propertyName Is "Phone" OrElse propertyName Is "AddressLine1" OrElse propertyName Is "City" OrElse propertyName Is "State" Then
				Return
			End If
			Dim prop = GetType(Customer).GetProperty(propertyName).PropertyType
			If prop.IsValueType OrElse prop Is GetType(String) Then
				MyBase.RadGridView1_SortChanged(sender, e)
			End If

		End Sub
		Protected Overrides Sub RefreshData(ByVal skip As Integer)
			data.Clear()

			Dim query = SortHelper.Sort(MainRepository.GetIndividualCustomers(), Me.gridControl.SortDescriptors)
			query = FilterHelper.Filter(query, Me.gridControl.FilterDescriptors)
			gridControl.RowCount = query.Count()

			Me.gridControl.MasterViewInfo.IsWaiting = True
			ExecuteQueryAsync(Of List(Of Customer))(Task.Run(Function() query.Skip(skip).Take(Me.gridControl.PageSize).ToList()), Me.Callback)

		End Sub

		Protected Overrides Function CreateWorkbook() As Workbook
			Dim workbook_Renamed As New Workbook()
			Dim worksheet_Renamed As Worksheet = workbook_Renamed.Worksheets.Add()

			' set header
			For i As Integer = 0 To columnNames.Count - 1
				Dim selection As CellSelection = worksheet_Renamed.Cells(0, i)
				selection.SetValue(columnNames(i))
			Next i

			For i As Integer = 0 To data.Count - 1
				Dim rowIndex As Integer = i + 1
				Dim selection As CellSelection = worksheet_Renamed.Cells(rowIndex, 0)
				selection.SetValue(data(i).AccountNumber)

				selection = worksheet_Renamed.Cells(rowIndex, 1)
				selection.SetValue(data(i).FirstName)

				selection = worksheet_Renamed.Cells(rowIndex, 2)
				selection.SetValue(data(i).LastName)

				selection = worksheet_Renamed.Cells(rowIndex, 3)
				selection.SetValue(data(i).EmailAddress)

				selection = worksheet_Renamed.Cells(rowIndex, 4)
				selection.SetValue(data(i).Phone)

				selection = worksheet_Renamed.Cells(rowIndex, 5)
				selection.SetValue(data(i).AddressLine1)

				selection = worksheet_Renamed.Cells(rowIndex, 6)
				selection.SetValue(data(i).City)

				selection = worksheet_Renamed.Cells(rowIndex, 7)
				selection.SetValue(data(i).State)

				selection = worksheet_Renamed.Cells(rowIndex, 8)
				selection.SetValue(data(i).ModifiedDate)

			Next i
			worksheet_Renamed.Columns(worksheet_Renamed.UsedCellRange).AutoFitWidth()
			worksheet_Renamed.Name = "Individuals"
			Return workbook_Renamed
		End Function
	End Class
End Namespace
