﻿Imports ERP.Repository
Imports ERP.Repository.Service
Imports System.Data.Services.Client
Imports System.IO
Imports System.Text
Imports System.Threading.Tasks
Imports Telerik.WinControls
Imports Telerik.WinControls.UI
Imports Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx
Imports Telerik.Windows.Documents.Spreadsheet.Model

Namespace ERP.Client
	Friend Class InventoriesControl
		Inherits BaseGridControl
        Private data As List(Of ProductInventory)
        Private privateCallback As Action(Of List(Of ProductInventory))
		Public Property Callback() As Action(Of List(Of ProductInventory))
			Get
				Return privateCallback
			End Get
			Private Set(ByVal value As Action(Of List(Of ProductInventory)))
				privateCallback = value
			End Set
		End Property

        Protected Overrides Sub Initialize()
            data = New List(Of ProductInventory)()
            Me.dataFormText = "Edit Product Inventory"
            columnTypes = New Type() {GetType(String), GetType(String), GetType(Short), GetType(String), GetType(Short), GetType(String), GetType(Decimal), GetType(Decimal), GetType(String), GetType(Byte), GetType(String)}
            AddHandler Me.gridControl.CreateCellElement, AddressOf GridControl_CreateCellElement
            Me.columnNames.Add("Product Number")
            Me.columnNames.Add("Product")
            Me.columnNames.Add("Quantity")
            Me.columnNames.Add("Color")
            Me.columnNames.Add("Stock Level")
            Me.columnNames.Add("Size")
            Me.columnNames.Add("Price")
            Me.columnNames.Add("Cost")
            Me.columnNames.Add("Shelf")
            Me.columnNames.Add("Bin")
            Me.columnNames.Add("Location")
            Me.gridControl.ColumnCount = columnNames.Count

            Me.gridControl.RowCount = MainRepository.Context.ProductInventories.Count()

            Me.Callback = New Action(Of List(Of ProductInventory))(Sub(query)
                                                                       Me.data = query
                                                                       Me.gridControl.MasterViewInfo.IsWaiting = False
                                                                       Me.gridControl.TableElement.SynchronizeRows()
                                                                   End Sub)

            Me.RefreshData(0)
            AddHandler Me.gridControl.SelectionChanged, AddressOf GridControl_SelectionChanged
            Me.gridControl.MasterViewInfo.SetColumnDataType(columnTypes)
            AddHandler Me.gridControl.CellFormatting, AddressOf GridControl_CellFormatting
        End Sub

        Private Sub GridControl_CellFormatting(ByVal sender As Object, ByVal e As VirtualGridCellElementEventArgs)
			If TypeOf e.CellElement Is VirtualGridFilterCellElement Then
				If e.CellElement.ColumnIndex = 0 OrElse e.CellElement.ColumnIndex = 1 OrElse e.CellElement.ColumnIndex = 3 OrElse e.CellElement.ColumnIndex = 4 OrElse e.CellElement.ColumnIndex = 5 OrElse e.CellElement.ColumnIndex = 6 OrElse e.CellElement.ColumnIndex = 7 OrElse e.CellElement.ColumnIndex = 10 Then
					e.CellElement.Enabled = False
				Else
					e.CellElement.ResetValue(RadItem.EnabledProperty, ValueResetFlags.Local)

				End If
			End If
			If TypeOf e.CellElement Is VirtualGridHeaderCellElement Then
				Dim cell = TryCast(e.CellElement, VirtualGridHeaderCellElement)
				If e.CellElement.ColumnIndex = 0 OrElse e.CellElement.ColumnIndex = 1 OrElse e.CellElement.ColumnIndex = 3 OrElse e.CellElement.ColumnIndex = 4 OrElse e.CellElement.ColumnIndex = 5 OrElse e.CellElement.ColumnIndex = 6 OrElse e.CellElement.ColumnIndex = 7 OrElse e.CellElement.ColumnIndex = 10 Then
					cell.Arrow.ShouldPaint = False
				Else
					cell.Arrow.ResetValue(LightVisualElement.ShouldPaintProperty, ValueResetFlags.Local)
				End If
			End If
		End Sub

		Private Sub GridControl_SelectionChanged(ByVal sender As Object, ByVal e As EventArgs)
			If Me.gridControl.CurrentCell IsNot Nothing AndAlso Me.gridControl.CurrentCell.RowIndex >= 0 Then
				currentItem = data(Me.gridControl.CurrentCell.RowIndex Mod Me.gridControl.PageSize)
			End If
		End Sub

		Private Sub GridControl_CreateCellElement(ByVal sender As Object, ByVal e As VirtualGridCreateCellEventArgs)
			If e.ColumnIndex = 3 AndAlso e.RowIndex >= 0 Then
				e.CellElement = New ColorCellElement()
			End If
		End Sub

		Protected Overrides Sub RadGridView1_CellValueNeeded(ByVal sender As Object, ByVal e As VirtualGridCellValueNeededEventArgs)
			MyBase.RadGridView1_CellValueNeeded(sender, e)
			If e.ColumnIndex < 0 Then
				Return
			End If
			If e.RowIndex < 0 Then
				e.FieldName = FieldsHelper.InventoriesFields(e.ColumnIndex)

			End If
			If e.RowIndex = RadVirtualGrid.HeaderRowIndex Then
				e.Value = columnNames(e.ColumnIndex)
			End If
			If e.RowIndex >= 0 AndAlso data.Count > 0 Then
				Dim rowData = TryCast(data(e.RowIndex Mod gridControl.PageSize), ProductInventory)

				Select Case e.ColumnIndex
					Case 0
						e.Value = rowData.ProductNumber
					Case 1
						e.Value = rowData.Product.Name
					Case 2
						e.Value = rowData.Quantity
					Case 3
						e.Value = rowData.Color
					Case 4
						e.Value = rowData.StockLevel
					Case 5
						e.Value = rowData.Size
					Case 6
						e.Value = rowData.Price
					Case 7
						e.Value = rowData.Cost
					Case 8
						e.Value = rowData.Shelf
					Case 9
						e.Value = rowData.Bin
					Case 10
						e.Value = rowData.Location.Name
				End Select
			End If
		End Sub
		Protected Overrides Sub RadGridView1_SortChanged(ByVal sender As Object, ByVal e As VirtualGridEventArgs)
			If e.ViewInfo.SortDescriptors.Count = 0 Then
				Return
			End If
			Dim propertyName = e.ViewInfo.SortDescriptors(0).PropertyName
			If propertyName Is "Color" OrElse propertyName Is "ProductNumber" OrElse propertyName Is "StockLevel" OrElse propertyName Is "Size" OrElse propertyName Is "Cost" OrElse propertyName Is "Price" OrElse propertyName Is "Location" Then
				Return
			End If
			Dim prop = GetType(ProductInventory).GetProperty(propertyName).PropertyType
			If prop.IsValueType OrElse prop Is GetType(String) Then
				MyBase.RadGridView1_SortChanged(sender, e)
			End If


		End Sub
		Protected Overrides Sub RefreshData(ByVal skip As Integer)
			data.Clear()
			Dim query = SortHelper.Sort(MainRepository.Context.ProductInventories.Expand(Function(p) p.Product).Expand(Function(p) p.Location), Me.gridControl.SortDescriptors)
			query = FilterHelper.Filter(query, Me.gridControl.FilterDescriptors)
			gridControl.RowCount = query.Count()

			Me.gridControl.MasterViewInfo.IsWaiting = True
			ExecuteQueryAsync(Of List(Of ProductInventory))(Task.Run(Function() query.Skip(skip).Take(Me.gridControl.PageSize).ToList()), Me.Callback)
		End Sub

		Protected Overrides Function CreateWorkbook() As Workbook
			Dim workbook_Renamed As New Workbook()
			Dim worksheet_Renamed As Worksheet = workbook_Renamed.Worksheets.Add()

			' set header
			For i As Integer = 0 To columnNames.Count - 1
				Dim selection As CellSelection = worksheet_Renamed.Cells(0, i)
				selection.SetValue(columnNames(i))
			Next i

			For i As Integer = 0 To data.Count - 1
				Dim rowIndex As Integer = i + 1
				Dim selection As CellSelection = worksheet_Renamed.Cells(rowIndex, 0)
				selection.SetValue(data(i).ProductNumber)

				selection = worksheet_Renamed.Cells(rowIndex, 1)
				selection.SetValue(data(i).Product.Name)

				selection = worksheet_Renamed.Cells(rowIndex, 2)
				selection.SetValue(data(i).Quantity)

				selection = worksheet_Renamed.Cells(rowIndex, 3)
				selection.SetValue(data(i).Color)

				selection = worksheet_Renamed.Cells(rowIndex, 4)
				selection.SetValue(data(i).StockLevel)

				selection = worksheet_Renamed.Cells(rowIndex, 5)
				selection.SetValue(data(i).Size)

				selection = worksheet_Renamed.Cells(rowIndex, 6)
				selection.SetValue(Convert.ToDouble(data(i).Price))

				selection = worksheet_Renamed.Cells(rowIndex, 7)
				selection.SetValue(Convert.ToDouble(data(i).Cost))

				selection = worksheet_Renamed.Cells(rowIndex, 8)
				selection.SetValue(data(i).Shelf)

				selection = worksheet_Renamed.Cells(rowIndex, 9)
				selection.SetValue(data(i).Bin)

				selection = worksheet_Renamed.Cells(rowIndex, 10)
				selection.SetValue(data(i).Location.Name)


			Next i
			worksheet_Renamed.Columns(worksheet_Renamed.UsedCellRange).AutoFitWidth()
			worksheet_Renamed.Name = "Inventories"
			Return workbook_Renamed
		End Function
	End Class
End Namespace
