﻿Imports ERP.Repository
Imports ERP.Repository.Service
Imports System.ComponentModel
Imports System.Data.Services.Client
Imports System.Text
Imports System.Threading.Tasks
Imports Telerik.WinControls.UI
Imports System.Data.Entity
Imports Telerik.WinControls.Export
Imports Telerik.Windows.Documents.Spreadsheet.Model
Imports Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx
Imports System.IO
Imports Telerik.WinControls

Namespace ERP.Client
	Friend Class OrdersControl
		Inherits BaseGridControl

        Private orderDetailsControl1 As OrderDetailsControl

        Private data As List(Of SalesOrderHeader)
        Private privateCallback As Action(Of List(Of SalesOrderHeader))
		Public Property Callback() As Action(Of List(Of SalesOrderHeader))
			Get
				Return privateCallback
			End Get
			Private Set(ByVal value As Action(Of List(Of SalesOrderHeader)))
				privateCallback = value
			End Set
		End Property

        Protected Overrides Sub Initialize()
            data = New List(Of SalesOrderHeader)()
            Me.dataFormText = "Edit Order"
            orderDetailsControl1 = New OrderDetailsControl()
            orderDetailsControl1.Parent = Me
            orderDetailsControl1.Dock = DockStyle.Bottom
            orderDetailsControl1.Height = 330
            orderDetailsControl1.Margin = New Padding(3)

            AddHandler Me.gridControl.SelectionChanged, AddressOf GridControl_SelectionChanged
            columnTypes = New Type() {GetType(String), GetType(String), GetType(Date), GetType(Boolean), GetType(String), GetType(Decimal), GetType(Decimal), GetType(Decimal), GetType(Date)}
            Me.gridControl.ColumnCount = 10

            Me.columnNames.Add("Order Number")
            Me.columnNames.Add("Customer")
            Me.columnNames.Add("Due Date")
            Me.columnNames.Add("Is Online Order")
            Me.columnNames.Add("Account Number")
            Me.columnNames.Add("Sub Total")
            Me.columnNames.Add("Tax Amount")
            Me.columnNames.Add("Freight")
            Me.columnNames.Add("Total Due")
            Me.columnNames.Add("Ship Method")

            Me.gridControl.RowCount = MainRepository.GetPurchaseOrders().Count()

            Me.Callback = New Action(Of List(Of SalesOrderHeader))(Sub(query)
                                                                       Me.data = query
                                                                       Me.gridControl.MasterViewInfo.IsWaiting = False
                                                                       Me.gridControl.TableElement.SynchronizeRows()
                                                                   End Sub)
            Me.gridControl.MasterViewInfo.IsWaiting = True
            Dim func = New Action(Of List(Of Product))(Sub(query) Me.RefreshData(0))

            ExecuteQueryAsync(Of List(Of Product))(Task.Run(Function() MainRepository.ProductsCache.ToList()), func)

            Me.gridControl.MasterViewInfo.SetColumnDataType(columnTypes)
            AddHandler Me.gridControl.CellFormatting, AddressOf GridControl_CellFormatting
        End Sub

        Private Sub GridControl_CellFormatting(ByVal sender As Object, ByVal e As VirtualGridCellElementEventArgs)
			If TypeOf e.CellElement Is VirtualGridFilterCellElement Then
				If e.CellElement.ColumnIndex = 1 OrElse e.CellElement.ColumnIndex = 9 Then
					e.CellElement.Enabled = False
				Else
					e.CellElement.ResetValue(RadItem.EnabledProperty, ValueResetFlags.Local)
				End If
			End If
			If TypeOf e.CellElement Is VirtualGridHeaderCellElement Then
				Dim cell = TryCast(e.CellElement, VirtualGridHeaderCellElement)
				If e.CellElement.ColumnIndex = 1 OrElse e.CellElement.ColumnIndex = 9 Then
					cell.Arrow.ShouldPaint = False
				Else
					cell.Arrow.ResetValue(LightVisualElement.ShouldPaintProperty, ValueResetFlags.Local)
				End If
			End If
		End Sub

		Private Sub GridControl_SelectionChanged(ByVal sender As Object, ByVal e As EventArgs)
			If Me.gridControl.CurrentCell IsNot Nothing AndAlso Me.gridControl.CurrentCell.RowIndex >= 0 Then
				Dim item = data(Me.gridControl.CurrentCell.RowIndex Mod gridControl.PageSize)
                orderDetailsControl1.Data = item
                currentItem = item
			End If

		End Sub

		Protected Overrides Sub RadGridView1_CellValueNeeded(ByVal sender As Object, ByVal e As VirtualGridCellValueNeededEventArgs)
			MyBase.RadGridView1_CellValueNeeded(sender, e)
			If e.ColumnIndex < 0 Then
				Return
			End If
			If e.RowIndex < 0 Then
				e.FieldName = FieldsHelper.OrdersFields(e.ColumnIndex)

			End If
			If e.RowIndex = RadVirtualGrid.HeaderRowIndex Then
				e.Value = columnNames(e.ColumnIndex)
			End If
			If e.RowIndex >= 0 AndAlso data.Count > 0 Then
				Dim rowData = TryCast(data(e.RowIndex Mod gridControl.PageSize), SalesOrderHeader)

				Select Case e.ColumnIndex
					Case 0
						e.Value = rowData.SalesOrderNumber
					Case 1
						e.Value = rowData.Customer.FirstName & " " & rowData.Customer.LastName
					Case 2
						e.Value = rowData.DueDate
						e.FormatString = "{0:MM/dd/yyyy}"
					Case 3
						e.Value = rowData.OnlineOrderFlag
					Case 4
						e.Value = rowData.AccountNumber
					Case 5
						e.Value = rowData.SubTotal
					Case 6
						e.Value = rowData.TaxAmt
					Case 7
						e.Value = rowData.Freight
					Case 8
						e.Value = rowData.TotalDue
					Case 9
						e.Value = rowData.ShipMethod.Name
				End Select
			End If
		End Sub
		Protected Overrides Sub RadGridView1_SortChanged(ByVal sender As Object, ByVal e As VirtualGridEventArgs)
			If e.ViewInfo.SortDescriptors.Count = 0 Then
				Return
			End If

			Dim propertyName = e.ViewInfo.SortDescriptors(0).PropertyName
			Dim prop = GetType(SalesOrderHeader).GetProperty(propertyName).PropertyType
			If prop.IsValueType OrElse prop Is GetType(String) Then
				MyBase.RadGridView1_SortChanged(sender, e)
			End If

		End Sub
		Protected Overrides Sub RefreshData(ByVal skip As Integer)
			data.Clear()

			Dim query = SortHelper.Sort(MainRepository.GetSalesOrders(), Me.gridControl.SortDescriptors)
			query = FilterHelper.Filter(query, Me.gridControl.FilterDescriptors)
			gridControl.RowCount = query.Count()

			Me.gridControl.MasterViewInfo.IsWaiting = True
			ExecuteQueryAsync(Of List(Of SalesOrderHeader))(Task.Run(Function() query.Skip(skip).Take(Me.gridControl.PageSize).ToList()), Me.Callback)
		End Sub

		Protected Overrides Function CreateWorkbook() As Workbook
			Dim workbook_Renamed As New Workbook()
			Dim worksheet_Renamed As Worksheet = workbook_Renamed.Worksheets.Add()

			' set header
			For i As Integer = 0 To columnNames.Count - 1
				Dim selection As CellSelection = worksheet_Renamed.Cells(0, i)
				selection.SetValue(columnNames(i))
			Next i

			For i As Integer = 0 To data.Count - 1
				Dim rowIndex As Integer = i + 1
				Dim selection As CellSelection = worksheet_Renamed.Cells(rowIndex, 0)
				selection.SetValue(data(i).PurchaseOrderNumber)

				selection = worksheet_Renamed.Cells(rowIndex, 1)
				selection.SetValue(data(i).Customer.FirstName & " " & data(i).Customer.LastName)

				selection = worksheet_Renamed.Cells(rowIndex, 2)
				selection.SetValue(data(i).DueDate)

				selection = worksheet_Renamed.Cells(rowIndex, 3)
				selection.SetValue(data(i).OnlineOrderFlag)

				selection = worksheet_Renamed.Cells(rowIndex, 4)
				selection.SetValue(data(i).AccountNumber)

				selection = worksheet_Renamed.Cells(rowIndex, 5)
				selection.SetValue(Convert.ToDouble(data(i).SubTotal))

				selection = worksheet_Renamed.Cells(rowIndex, 6)
				selection.SetValue(Convert.ToDouble(data(i).TaxAmt))

				selection = worksheet_Renamed.Cells(rowIndex, 7)
				selection.SetValue(Convert.ToDouble(data(i).Freight))

				selection = worksheet_Renamed.Cells(rowIndex, 8)
				selection.SetValue(Convert.ToDouble(data(i).TotalDue))

				selection = worksheet_Renamed.Cells(rowIndex, 8)
				selection.SetValue(data(i).ShipMethod.Name)
			Next i
			worksheet_Renamed.Columns(worksheet_Renamed.UsedCellRange).AutoFitWidth()
			worksheet_Renamed.Name = "Orders"
			Return workbook_Renamed


		End Function
	End Class
End Namespace
