﻿Imports ERP.Repository
Imports ERP.Repository.Service
Imports System.ComponentModel
Imports System.Data.Services.Client
Imports System.Text
Imports System.Threading.Tasks
Imports Telerik.WinControls.UI
Imports System.Data.Entity
Imports Telerik.WinControls.Export
Imports Telerik.Windows.Documents.Spreadsheet.Model
Imports Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx
Imports System.IO
Imports Telerik.WinControls

Namespace ERP.Client
	Friend Class PurchasesControl
		Inherits BaseGridControl
        Private data As List(Of PurchaseOrderHeader)

        Private detailsData As New List(Of PurchaseOrderDetail)()

		Private privateCallback As Action(Of List(Of PurchaseOrderHeader))
		Public Property Callback() As Action(Of List(Of PurchaseOrderHeader))
			Get
				Return privateCallback
			End Get
			Private Set(ByVal value As Action(Of List(Of PurchaseOrderHeader)))
				privateCallback = value
			End Set
		End Property

        Private detailsViewColumnNames As List(Of String)

        Protected Overrides Sub Initialize()
            detailsViewColumnNames = New List(Of String)()
            data = New List(Of PurchaseOrderHeader)()

            Me.dataFormText = "Edit Purchase"
            Me.gridControl.ColumnCount = 10
            columnTypes = New Type() {GetType(String), GetType(Date), GetType(Date), GetType(Decimal), GetType(Decimal), GetType(Decimal), GetType(Decimal), GetType(Date), GetType(String), GetType(String)}

            Me.columnNames.Add("Order Status")
            Me.columnNames.Add("Order Date")
            Me.columnNames.Add("Ship Date")
            Me.columnNames.Add("Sub Total")
            Me.columnNames.Add("Tax Amount")
            Me.columnNames.Add("Freight")
            Me.columnNames.Add("Total Due")
            Me.columnNames.Add("Modified Date")
            Me.columnNames.Add("Ship Method")
            Me.columnNames.Add("Vendor")

            Me.gridControl.RowCount = MainRepository.GetPurchaseOrders().Count()

            Me.Callback = New Action(Of List(Of PurchaseOrderHeader))(Sub(query)
                                                                          Me.data = query
                                                                          Me.gridControl.MasterViewInfo.IsWaiting = False
                                                                          Me.gridControl.TableElement.SynchronizeRows()
                                                                      End Sub)

            Me.RefreshData(0)

            Me.detailsViewColumnNames.Add("Product")
            Me.detailsViewColumnNames.Add("Quantity")
            Me.detailsViewColumnNames.Add("Received Quantity")
            Me.detailsViewColumnNames.Add("Rejected Quantity")
            Me.detailsViewColumnNames.Add("Stocked Quantity")
            Me.detailsViewColumnNames.Add("Price per Unit")
            Me.detailsViewColumnNames.Add("Total Price")
            Me.detailsViewColumnNames.Add("Due Date")
            Me.detailsViewColumnNames.Add("Modified Date")

            AddHandler Me.gridControl.RowExpanding, AddressOf GridControl_RowExpanding
            AddHandler Me.gridControl.QueryHasChildRows, AddressOf GridControl_QueryHasChildRows
            AddHandler Me.gridControl.SelectionChanged, AddressOf GridControl_SelectionChanged

            AddHandler Me.gridControl.CellFormatting, AddressOf GridControl_CellFormatting
            Me.gridControl.MasterViewInfo.SetColumnDataType(columnTypes)
            AddHandler Me.gridControl.RowFormatting, AddressOf GridControl_RowFormatting
        End Sub

        Private Sub GridControl_RowFormatting(ByVal sender As Object, ByVal e As VirtualGridRowElementEventArgs)
			If e.ViewInfo IsNot Me.gridControl.MasterViewInfo AndAlso TypeOf e.RowElement Is VirtualGridFilterRowElement Then
				e.RowElement.Visibility = ElementVisibility.Collapsed
			Else
				e.RowElement.ResetValue(RadItem.VisibilityProperty, ValueResetFlags.Local)
			End If
		End Sub

		Private Sub GridControl_CellFormatting(ByVal sender As Object, ByVal e As VirtualGridCellElementEventArgs)
			If TypeOf e.CellElement Is VirtualGridFilterCellElement Then
				If e.CellElement.ColumnIndex = 0 OrElse e.CellElement.ColumnIndex = 9 OrElse e.CellElement.ColumnIndex = 8 Then
					e.CellElement.Enabled = False
				Else
					e.CellElement.ResetValue(RadItem.EnabledProperty, ValueResetFlags.Local)

				End If
			End If
			If TypeOf e.CellElement Is VirtualGridHeaderCellElement Then
				Dim cell = TryCast(e.CellElement, VirtualGridHeaderCellElement)
				If e.CellElement.ColumnIndex = 0 OrElse e.CellElement.ColumnIndex = 9 OrElse e.CellElement.ColumnIndex = 8 Then
					cell.Arrow.ShouldPaint = False
				Else
					cell.Arrow.ResetValue(LightVisualElement.ShouldPaintProperty, ValueResetFlags.Local)
				End If
			End If
		End Sub

		Private Sub GridControl_SelectionChanged(ByVal sender As Object, ByVal e As EventArgs)
			If Me.gridControl.CurrentCell IsNot Nothing AndAlso Me.gridControl.CurrentCell.RowIndex >= 0 Then
				currentItem = data(Me.gridControl.CurrentCell.RowIndex Mod Me.gridControl.PageSize)
			End If
		End Sub

		Private Sub GridControl_QueryHasChildRows(ByVal sender As Object, ByVal e As VirtualGridQueryHasChildRowsEventArgs)
			e.HasChildRows = e.ViewInfo Is Me.gridControl.MasterViewInfo
		End Sub

		Private Sub GridControl_RowExpanding(ByVal sender As Object, ByVal e As VirtualGridRowExpandingEventArgs)
			detailsData = data(e.RowIndex Mod gridControl.PageSize).PurchaseOrderDetails.ToList()
			e.ChildViewInfo.RowCount = detailsData.Count
			e.ChildViewInfo.ColumnCount = detailsViewColumnNames.Count
		End Sub

		Protected Overrides Sub RadGridView1_CellValueNeeded(ByVal sender As Object, ByVal e As VirtualGridCellValueNeededEventArgs)
			MyBase.RadGridView1_CellValueNeeded(sender, e)
			If e.ColumnIndex < 0 Then
				Return
			End If

			If e.ViewInfo Is Me.gridControl.MasterViewInfo Then
				If e.RowIndex < 0 Then
					e.FieldName = FieldsHelper.PurchaseOrderHeaderFields(e.ColumnIndex)
				End If
				If e.RowIndex = RadVirtualGrid.HeaderRowIndex Then
					e.Value = columnNames(e.ColumnIndex)
					e.FieldName = FieldsHelper.PurchaseOrderHeaderFields(e.ColumnIndex)
				End If
				If e.RowIndex >= 0 AndAlso data.Count > 0 Then
					Dim rowData = TryCast(data(e.RowIndex Mod gridControl.PageSize), PurchaseOrderHeader)

					Select Case e.ColumnIndex
						Case 0
							e.Value = rowData.OrderStatus
						Case 1
							e.Value = rowData.OrderDate
						Case 2
							e.Value = rowData.ShipDate
						Case 3
							e.Value = rowData.SubTotal
						Case 4
							e.Value = rowData.TaxAmt
						Case 5
							e.Value = rowData.Freight
						Case 6
							e.Value = rowData.TotalDue
						Case 7
							e.Value = rowData.ModifiedDate
							e.FormatString = "{0:MM/dd/yyyy}"
						Case 8
							e.Value = rowData.ShipMethod
						Case 9
							e.Value = rowData.Vendor.Name
					End Select
				End If
			Else
				If e.RowIndex = RadVirtualGrid.HeaderRowIndex Then
					e.Value = detailsViewColumnNames(e.ColumnIndex)
				End If
				If e.RowIndex >= 0 AndAlso detailsData.Count > 0 Then
					Dim rowData = TryCast(detailsData(e.RowIndex), PurchaseOrderDetail)

					Select Case e.ColumnIndex
						Case 0
							e.Value = rowData.Product
						Case 1
							e.Value = rowData.OrderQty
						Case 2
							e.Value = rowData.ReceivedQty
						Case 3
							e.Value = rowData.RejectedQty
						Case 4
							e.Value = rowData.StockedQty
						Case 5
							e.Value = rowData.UnitPrice
						Case 6
							e.Value = rowData.LineTotal
						Case 7
							e.Value = rowData.DueDate
							e.FormatString = "{0:MM/dd/yyyy}"
						Case 8
							e.Value = rowData.ModifiedDate
							e.FormatString = "{0:MM/dd/yyyy}"
					End Select
				End If


			End If
		End Sub
		Protected Overrides Sub RadGridView1_SortChanged(ByVal sender As Object, ByVal e As VirtualGridEventArgs)
			If e.ViewInfo.SortDescriptors.Count = 0 Then
				Return
			End If

			Dim propertyName = e.ViewInfo.SortDescriptors(0).PropertyName
			If propertyName Is "OrderStatus" Then
				Return
			End If
			Dim prop = GetType(PurchaseOrderHeader).GetProperty(propertyName).PropertyType
			If prop.IsValueType OrElse prop Is GetType(String) Then
				MyBase.RadGridView1_SortChanged(sender, e)
			End If

		End Sub
        Protected Overrides Sub RefreshData(ByVal skip As Integer)
            If data IsNot Nothing Then
                data.Clear()
            End If

            Dim query = SortHelper.Sort(MainRepository.GetPurchaseOrders(), Me.gridControl.SortDescriptors)
            query = FilterHelper.Filter(query, Me.gridControl.FilterDescriptors)
            gridControl.RowCount = query.Count()


            Me.gridControl.MasterViewInfo.IsWaiting = True
            ExecuteQueryAsync(Of List(Of PurchaseOrderHeader))(Task.Run(Function() query.Skip(skip).Take(Me.gridControl.PageSize).ToList()), Me.Callback)
        End Sub

        Protected Overrides Function CreateWorkbook() As Workbook
			Dim workbook_Renamed As New Workbook()
			Dim worksheet_Renamed As Worksheet = workbook_Renamed.Worksheets.Add()

			' set header
			For i As Integer = 0 To columnNames.Count - 1
				Dim selection As CellSelection = worksheet_Renamed.Cells(0, i)
				selection.SetValue(columnNames(i))
			Next i

			For i As Integer = 0 To data.Count - 1
				Dim rowIndex As Integer = i + 1
				Dim selection As CellSelection = worksheet_Renamed.Cells(rowIndex, 0)
				selection.SetValue(data(i).OrderStatus)

				selection = worksheet_Renamed.Cells(rowIndex, 1)
				selection.SetValue(data(i).OrderDate)

				selection = worksheet_Renamed.Cells(rowIndex, 2)
				selection.SetValue(data(i).ShipDate.Value)

				selection = worksheet_Renamed.Cells(rowIndex, 3)
				selection.SetValue(Convert.ToDouble(data(i).SubTotal))

				selection = worksheet_Renamed.Cells(rowIndex, 4)
				selection.SetValue(Convert.ToDouble(data(i).TaxAmt))

				selection = worksheet_Renamed.Cells(rowIndex, 5)
				selection.SetValue(Convert.ToDouble(data(i).Freight))

				selection = worksheet_Renamed.Cells(rowIndex, 6)
				selection.SetValue(Convert.ToDouble(data(i).TotalDue))

				selection = worksheet_Renamed.Cells(rowIndex, 7)
				selection.SetValue(data(i).ModifiedDate)

				selection = worksheet_Renamed.Cells(rowIndex, 8)
				selection.SetValue(data(i).ShipMethod.Name)

				selection = worksheet_Renamed.Cells(rowIndex, 9)
				selection.SetValue(data(i).Vendor.Name)

			Next i
			worksheet_Renamed.Columns(worksheet_Renamed.UsedCellRange).AutoFitWidth()
			worksheet_Renamed.Name = "Purchases"
			Return workbook_Renamed
		End Function
	End Class
End Namespace