﻿Imports ERP.Repository
Imports ERP.Repository.Service
Imports System.ComponentModel
Imports System.Data.Services.Client
Imports System.Text
Imports System.Threading.Tasks
Imports Telerik.WinControls.UI
Imports System.Data.Entity
Imports Telerik.WinControls.Export
Imports Telerik.Windows.Documents.Spreadsheet.Model
Imports Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx
Imports System.IO

Namespace ERP.Client
	Friend Class VendorsControl
		Inherits BaseGridControl
        Private data As List(Of Vendor)
        Private privateCallback As Action(Of List(Of Vendor))
		Public Property Callback() As Action(Of List(Of Vendor))
			Get
				Return privateCallback
			End Get
			Private Set(ByVal value As Action(Of List(Of Vendor)))
				privateCallback = value
			End Set
		End Property

        Protected Overrides Sub Initialize()
            data = New List(Of Vendor)()
            Me.dataFormText = "Edit Supplier"
            columnTypes = New Type() {GetType(String), GetType(String), GetType(Byte), GetType(String), GetType(String), GetType(String), GetType(Date)}
            Me.gridControl.ColumnCount = 7
            AddHandler Me.gridControl.CreateCellElement, AddressOf GridControl_CreateCellElement

            Me.columnNames.Add("Account Number")
            Me.columnNames.Add("Name")
            Me.columnNames.Add("Credit Rating")
            Me.columnNames.Add("Preference")
            Me.columnNames.Add("ActiveStaus")
            Me.columnNames.Add("URL")
            Me.columnNames.Add("Modified Date")


            Me.gridControl.RowCount = MainRepository.Context.Vendors.Count()

            Me.Callback = New Action(Of List(Of Vendor))(Sub(query)
                                                             Me.data = query
                                                             Me.gridControl.MasterViewInfo.IsWaiting = False
                                                             Me.gridControl.TableElement.SynchronizeRows()
                                                         End Sub)


            Me.RefreshData(0)
            AddHandler Me.gridControl.CellFormatting, AddressOf GridControl_CellFormatting
            AddHandler Me.gridControl.SelectionChanged, AddressOf GridControl_SelectionChanged
            Me.gridControl.MasterViewInfo.SetColumnDataType(columnTypes)
        End Sub

        Private Sub GridControl_SelectionChanged(ByVal sender As Object, ByVal e As EventArgs)
			If Me.gridControl.CurrentCell IsNot Nothing AndAlso Me.gridControl.CurrentCell.RowIndex >= 0 Then
				currentItem = data(Me.gridControl.CurrentCell.RowIndex Mod Me.gridControl.PageSize)
			End If
		End Sub

		Private Sub GridControl_CreateCellElement(ByVal sender As Object, ByVal e As VirtualGridCreateCellEventArgs)
			If e.ColumnIndex = 2 AndAlso e.RowIndex >= 0 Then
				e.CellElement = New RatingCellElement()
			End If
		End Sub

		Private Sub GridControl_CellFormatting(ByVal sender As Object, ByVal e As VirtualGridCellElementEventArgs)
			If e.CellElement.RowIndex < 0 Then
				Return
			End If
			If e.CellElement.ColumnIndex = 4 AndAlso e.CellElement.Value IsNot Nothing Then
				Dim value = CBool(e.CellElement.Value)
				If value Then
                    e.CellElement.ForeColor = System.Drawing.Color.Green
                    e.CellElement.Text = "Active"
				Else
                    e.CellElement.ForeColor = System.Drawing.Color.Red
                    e.CellElement.Text = "Inactive"
				End If

			ElseIf e.CellElement.ColumnIndex = 3 AndAlso e.CellElement.Value IsNot Nothing Then
				Dim value = CBool(e.CellElement.Value)
				If value Then
                    e.CellElement.ForeColor = System.Drawing.Color.Green
                    e.CellElement.Text = "Preferred"
				Else
                    e.CellElement.ForeColor = System.Drawing.Color.Gold
                    e.CellElement.Text = "Not Preferred"
				End If

			End If
		End Sub

		Protected Overrides Sub RadGridView1_CellValueNeeded(ByVal sender As Object, ByVal e As VirtualGridCellValueNeededEventArgs)
			MyBase.RadGridView1_CellValueNeeded(sender, e)
			If e.ColumnIndex < 0 Then
				Return
			End If
			If e.RowIndex < 0 Then
				e.FieldName = FieldsHelper.VendorFields(e.ColumnIndex)
			End If
			If e.RowIndex = RadVirtualGrid.HeaderRowIndex Then
				e.Value = columnNames(e.ColumnIndex)
				e.FieldName = FieldsHelper.VendorFields(e.ColumnIndex)
			End If
			If e.RowIndex >= 0 AndAlso data.Count > 0 Then
				If (e.RowIndex Mod gridControl.PageSize) >= data.Count Then
					Return
				End If
				Dim rowData = TryCast(data(e.RowIndex Mod gridControl.PageSize), Vendor)

				Select Case e.ColumnIndex
					Case 0
						e.Value = rowData.AccountNumber
					Case 1
						e.Value = rowData.Name
					Case 2
						e.Value = rowData.CreditRating
					Case 3
						e.Value = rowData.PreferredVendorStatus
					Case 4
						e.Value = rowData.ActiveFlag
					Case 5
						e.Value = rowData.PurchasingWebServiceURL
					Case 6
						e.Value = rowData.ModifiedDate
						e.FormatString = "{0:MM/dd/yyyy}"
				End Select
			End If
		End Sub
		Protected Overrides Sub RadGridView1_SortChanged(ByVal sender As Object, ByVal e As VirtualGridEventArgs)
			If e.ViewInfo.SortDescriptors.Count = 0 Then
				Return
			End If

			Dim propertyName = e.ViewInfo.SortDescriptors(0).PropertyName
			Dim prop = GetType(Vendor).GetProperty(propertyName).PropertyType

			If prop.IsValueType OrElse prop Is GetType(String) Then
				MyBase.RadGridView1_SortChanged(sender, e)
			End If

		End Sub

		Protected Overrides Sub RefreshData(ByVal skip As Integer)
			data.Clear()
			Dim query = SortHelper.Sort(MainRepository.Context.Vendors, Me.gridControl.SortDescriptors)
			query = FilterHelper.Filter(query, Me.gridControl.FilterDescriptors)
			gridControl.RowCount = query.Count()

			Me.gridControl.MasterViewInfo.IsWaiting = True
			ExecuteQueryAsync(Of List(Of Vendor))(Task.Run(Function() query.Skip(skip).Take(Me.gridControl.PageSize).ToList()), Me.Callback)
		End Sub

		Protected Overrides Function CreateWorkbook() As Workbook
			Dim workbook_Renamed As New Workbook()
			Dim worksheet_Renamed As Worksheet = workbook_Renamed.Worksheets.Add()

			' set header
			For i As Integer = 0 To columnNames.Count - 1
				Dim selection As CellSelection = worksheet_Renamed.Cells(0, i)
				selection.SetValue(columnNames(i))
			Next i

			For i As Integer = 0 To data.Count - 1
				Dim rowIndex As Integer = i + 1
				Dim selection As CellSelection = worksheet_Renamed.Cells(rowIndex, 0)
				selection.SetValue(data(i).AccountNumber)

				selection = worksheet_Renamed.Cells(rowIndex, 1)
				selection.SetValue(data(i).Name)

				selection = worksheet_Renamed.Cells(rowIndex, 2)
				selection.SetValue(data(i).CreditRating)

				selection = worksheet_Renamed.Cells(rowIndex, 3)
				selection.SetValue(data(i).PreferredVendorStatus)

				selection = worksheet_Renamed.Cells(rowIndex, 4)
				selection.SetValue(data(i).ActiveFlag)

				selection = worksheet_Renamed.Cells(rowIndex, 5)
				selection.SetValue(data(i).PurchasingWebServiceURL)

				selection = worksheet_Renamed.Cells(rowIndex, 6)
				selection.SetValue(data(i).ModifiedDate)
			Next i
			worksheet_Renamed.Columns(worksheet_Renamed.UsedCellRange).AutoFitWidth()
			worksheet_Renamed.Name = "Suppliers"
			Return workbook_Renamed
		End Function
	End Class
End Namespace

