﻿Imports ERP.Repository
Imports ERP.Repository.Service
Imports System.ComponentModel
Imports System.Data.Services.Client
Imports System.Text
Imports System.Threading.Tasks
Imports Telerik.WinControls.UI
Imports System.Data.Entity
Imports Telerik.WinControls.Export
Imports Telerik.Windows.Documents.Spreadsheet.Model
Imports Telerik.Windows.Documents.Spreadsheet.FormatProviders.OpenXml.Xlsx
Imports System.IO
Imports Telerik.WinControls

Namespace ERP.Client
	Friend Class WorkOrdersControl
		Inherits BaseGridControl
        Private data As List(Of WorkOrder)
        Private privateCallback As Action(Of List(Of WorkOrder))
		Public Property Callback() As Action(Of List(Of WorkOrder))
			Get
				Return privateCallback
			End Get
			Private Set(ByVal value As Action(Of List(Of WorkOrder)))
				privateCallback = value
			End Set
		End Property

        Protected Overrides Sub Initialize()
            data = New List(Of WorkOrder)()

            Me.dataFormText = "Edit Work Order"
            Me.gridControl.ColumnCount = 8
            columnTypes = New Type() {GetType(String), GetType(Integer), GetType(Integer), GetType(Short), GetType(Date), GetType(Date), GetType(Date), GetType(Date)}
            Me.columnNames.Add("Product")
            Me.columnNames.Add("Quantity")
            Me.columnNames.Add("Stocked")
            Me.columnNames.Add("Scrapped")
            Me.columnNames.Add("Start Date")
            Me.columnNames.Add("End Date")
            Me.columnNames.Add("Due Date")
            Me.columnNames.Add("Modified Date")

            Me.gridControl.RowCount = MainRepository.Context.WorkOrders.Count()

            Me.Callback = New Action(Of List(Of WorkOrder))(Sub(query)
                                                                Me.data = query
                                                                Me.gridControl.MasterViewInfo.IsWaiting = False
                                                                Me.gridControl.TableElement.SynchronizeRows()
                                                            End Sub)
            Me.RefreshData(0)

            AddHandler Me.gridControl.SelectionChanged, AddressOf GridControl_SelectionChanged
            Me.gridControl.MasterViewInfo.SetColumnDataType(columnTypes)
            AddHandler Me.gridControl.CellFormatting, AddressOf GridControl_CellFormatting
        End Sub

        Private Sub GridControl_CellFormatting(ByVal sender As Object, ByVal e As VirtualGridCellElementEventArgs)
			If TypeOf e.CellElement Is VirtualGridFilterCellElement Then
				If e.CellElement.ColumnIndex = 0 Then
					e.CellElement.Enabled = False
				Else
					e.CellElement.ResetValue(RadItem.EnabledProperty, ValueResetFlags.Local)
				End If
			End If
			If TypeOf e.CellElement Is VirtualGridHeaderCellElement Then
				Dim cell = TryCast(e.CellElement, VirtualGridHeaderCellElement)
				If e.CellElement.ColumnIndex = 0 Then
					cell.Arrow.ShouldPaint = False
				Else
					cell.Arrow.ResetValue(LightVisualElement.ShouldPaintProperty, ValueResetFlags.Local)
				End If
			End If
		End Sub

		Private Sub GridControl_SelectionChanged(ByVal sender As Object, ByVal e As EventArgs)
			If Me.gridControl.CurrentCell IsNot Nothing AndAlso Me.gridControl.CurrentCell.RowIndex >= 0 Then
				currentItem = data(Me.gridControl.CurrentCell.RowIndex Mod Me.gridControl.PageSize)
			End If

		End Sub

		Protected Overrides Sub RadGridView1_CellValueNeeded(ByVal sender As Object, ByVal e As VirtualGridCellValueNeededEventArgs)
			MyBase.RadGridView1_CellValueNeeded(sender, e)
			If e.ColumnIndex < 0 Then
				Return
			End If

			If e.RowIndex < 0 Then
				e.FieldName = FieldsHelper.WorkOrdersFields(e.ColumnIndex)

			End If
			If e.RowIndex = RadVirtualGrid.HeaderRowIndex Then
				e.Value = columnNames(e.ColumnIndex)
			End If
			If e.RowIndex >= 0 AndAlso data.Count > 0 Then
				Dim rowData = TryCast(data(e.RowIndex Mod gridControl.PageSize), WorkOrder)

				Select Case e.ColumnIndex
					Case 0
						e.Value = rowData.Product.Name
					Case 1
						e.Value = rowData.OrderQty
					Case 2
						e.Value = rowData.StockedQty
					Case 3
						e.Value = rowData.ScrappedQty
					Case 4
						e.Value = rowData.StartDate
						e.FormatString = "{0:MM/dd/yyyy}"
					Case 5
						e.Value = rowData.EndDate
						e.FormatString = "{0:MM/dd/yyyy}"
					Case 6
						e.Value = rowData.DueDate
						e.FormatString = "{0:MM/dd/yyyy}"
					Case 7
						e.Value = rowData.ModifiedDate
						e.FormatString = "{0:MM/dd/yyyy}"

				End Select
			End If
		End Sub
		Protected Overrides Sub RadGridView1_SortChanged(ByVal sender As Object, ByVal e As VirtualGridEventArgs)
			If e.ViewInfo.SortDescriptors.Count = 0 Then
				Return
			End If

			Dim propertyName = e.ViewInfo.SortDescriptors(0).PropertyName
			Dim prop = GetType(WorkOrder).GetProperty(propertyName).PropertyType
			If prop.IsValueType OrElse prop Is GetType(String) Then
				MyBase.RadGridView1_SortChanged(sender, e)
			End If

		End Sub
		Protected Overrides Sub RefreshData(ByVal skip As Integer)
			data.Clear()

			Dim query = SortHelper.Sort(MainRepository.Context.WorkOrders.Expand(Function(p) p.Product), Me.gridControl.SortDescriptors)
			query = FilterHelper.Filter(query, Me.gridControl.FilterDescriptors)
			gridControl.RowCount = query.Count()

			Me.gridControl.MasterViewInfo.IsWaiting = True
			ExecuteQueryAsync(Of List(Of WorkOrder))(Task.Run(Function() query.Skip(skip).Take(Me.gridControl.PageSize).ToList()), Me.Callback)

		End Sub

		Protected Overrides Function CreateWorkbook() As Workbook
			Dim workbook_Renamed As New Workbook()
			Dim worksheet_Renamed As Worksheet = workbook_Renamed.Worksheets.Add()

			' set header
			For i As Integer = 0 To columnNames.Count - 1
				Dim selection As CellSelection = worksheet_Renamed.Cells(0, i)
				selection.SetValue(columnNames(i))
			Next i

			For i As Integer = 0 To data.Count - 1
				Dim rowIndex As Integer = i + 1
				Dim selection As CellSelection = worksheet_Renamed.Cells(rowIndex, 0)
				selection.SetValue(data(i).Product.Name)

				selection = worksheet_Renamed.Cells(rowIndex, 1)
				selection.SetValue(data(i).OrderQty)

				selection = worksheet_Renamed.Cells(rowIndex, 2)
				selection.SetValue(data(i).StockedQty)

				selection = worksheet_Renamed.Cells(rowIndex, 3)
				selection.SetValue(data(i).ScrappedQty)

				selection = worksheet_Renamed.Cells(rowIndex, 4)
				selection.SetValue(data(i).StartDate)

				selection = worksheet_Renamed.Cells(rowIndex, 5)
				selection.SetValue(data(i).EndDate.Value)

				selection = worksheet_Renamed.Cells(rowIndex, 6)
				selection.SetValue(data(i).DueDate)

				selection = worksheet_Renamed.Cells(rowIndex, 7)
				selection.SetValue(data(i).ModifiedDate)

			Next i
			worksheet_Renamed.Columns(worksheet_Renamed.UsedCellRange).AutoFitWidth()
			worksheet_Renamed.Name = "Work Orders"
			Return workbook_Renamed
		End Function
	End Class
End Namespace
