﻿Imports ERP.Repository
Imports System.Data.Services.Client
Imports System.Linq.Expressions
Imports System.Reflection
Imports System.Text
Imports System.Threading.Tasks
Imports Telerik.WinControls.Data

Namespace ERP.Client
	Public NotInheritable Class FilterHelper
		Private Shared ReadOnly StringStartsWithMethodInfo As MethodInfo = GetType(String).GetMethod("StartsWith", { GetType(String) })
		Private Shared ReadOnly StringEndsWithMethodInfo As MethodInfo = GetType(String).GetMethod("EndsWith", { GetType(String) })
		Private Shared ReadOnly StringContainsMethodInfo As MethodInfo = GetType(String).GetMethod("Contains", { GetType(String) })

		Private Sub New()
		End Sub
		Public Shared Function Filter(Of T As ISavableObject)(ByVal queryable As DataServiceQuery(Of T), ByVal filters As FilterDescriptorCollection) As DataServiceQuery(Of T)
			Dim query As DataServiceQuery(Of T) = queryable

			If filters IsNot Nothing AndAlso filters.Count > 0 Then

				Dim predicate As Expression = FilterHelper.BuildFilter(query.ElementType, filters)
				Dim exp = Expression.Call(GetType(Queryable), "Where", { query.ElementType }, query.Expression, Expression.Quote(predicate))
				query = TryCast(query.Provider.CreateQuery(exp), DataServiceQuery(Of T))
			End If

			Return query
		End Function

		Private Shared Function BuildFilter(ByVal elementType As Type, ByVal filters As FilterDescriptorCollection) As LambdaExpression
			Dim pe As ParameterExpression = Expression.Parameter(elementType, "fltr")
			Dim expression_Renamed As Expression = Nothing

			For Each filter As FilterDescriptor In filters
				Dim pi As PropertyInfo = elementType.GetProperty(filter.PropertyName)
				If filter.Value Is Nothing Then


					If pi.PropertyType.IsValueType Then
						filter.Value = Activator.CreateInstance(pi.PropertyType)
					End If
				End If
				If filter.GetType() IsNot pi.PropertyType Then
					filter.Value = Convert.ChangeType(filter.Value, pi.PropertyType)
				End If

				Dim value As Expression = Expression.Constant(filter.Value)
				Dim [property] As Expression = Expression.Property(pe, filter.PropertyName)
				Dim comparison As Expression = Nothing

				Select Case filter.Operator
					Case FilterOperator.None
						comparison = Expression.Empty()
					Case FilterOperator.IsLessThan
						comparison = Expression.LessThan([property], value)
					Case FilterOperator.IsLessThanOrEqualTo
						comparison = Expression.LessThanOrEqual([property], value)
					Case FilterOperator.IsEqualTo, FilterOperator.IsLike
						comparison = Expression.Equal([property], value)
					Case FilterOperator.IsNotEqualTo, FilterOperator.IsNotLike
						comparison = Expression.NotEqual([property], value)
					Case FilterOperator.IsGreaterThanOrEqualTo
						comparison = Expression.GreaterThanOrEqual([property], value)
					Case FilterOperator.IsGreaterThan
						comparison = Expression.GreaterThan([property], value)
					Case FilterOperator.StartsWith
						comparison = FilterHelper.GenerateStringMethodCall(StringStartsWithMethodInfo, [property], value)
					Case FilterOperator.EndsWith
						comparison = FilterHelper.GenerateStringMethodCall(StringEndsWithMethodInfo, [property], value)
					Case FilterOperator.Contains, FilterOperator.IsContainedIn
						comparison = FilterHelper.GenerateStringMethodCall(StringContainsMethodInfo, [property], value)
					Case FilterOperator.NotContains, FilterOperator.IsNotContainedIn
						comparison = Expression.Not(FilterHelper.GenerateStringMethodCall(StringContainsMethodInfo, [property], value))
					Case FilterOperator.IsNull
						comparison = Expression.Equal([property], Expression.Constant(Nothing))
					Case FilterOperator.IsNotNull
						comparison = Expression.NotEqual([property], Expression.Constant(Nothing))
				End Select

				If expression_Renamed Is Nothing Then
					expression_Renamed = comparison

					Continue For
				End If

				If filters.LogicalOperator = FilterLogicalOperator.And Then
					expression_Renamed = Expression.AndAlso(expression_Renamed, comparison)
				Else
					expression_Renamed = Expression.OrElse(expression_Renamed, comparison)
				End If
			Next filter

			Return Expression.Lambda(expression_Renamed, pe)
		End Function

		Private Shared Function GenerateStringMethodCall(ByVal methodInfo_Renamed As MethodInfo, ByVal left As Expression, ByVal right As Expression) As Expression
			If methodInfo_Renamed.IsStatic Then
				Return Expression.Call(methodInfo_Renamed, { left, right })
			End If

			If IsNullLiteral(left) Then
				left = Expression.Constant(String.Empty)
			End If

			Return Expression.Call(left, methodInfo_Renamed, right)
		End Function

		Private Shared Function IsNullLiteral(ByVal expression_Renamed As Expression) As Boolean
			Dim constantExpression_Renamed As ConstantExpression = TryCast(expression_Renamed, ConstantExpression)

			If constantExpression_Renamed IsNot Nothing Then
				Return constantExpression_Renamed.Value Is Nothing
			End If

			Return False
		End Function
	End Class
End Namespace
