﻿Imports System.Text
Imports System.Threading.Tasks
Imports Telerik.WinControls.UI
Imports Telerik.WinControls.UI.Map.Bing

Namespace ERP.Client
	Public Class MapLocationHelper
		Private Const VEKey As String = "AqaPuZWytKRUA8Nm5nqvXHWGL8BDCXvK8onCl2PkC581Zp3T_fYAQBiwIphJbRAK"
		Private map As RadMap
		Private provider As BingRestMapProvider
		Private visualizationLayer As MapLayer
		Private address As ERP.Repository.Service.Address
		Private isAddtionalSearch As Boolean


		Public Sub New(ByVal map As RadMap)
			Me.map = map
			Me.InitializeMapProvider()
		End Sub

		Public Sub UpdateMap(ByVal address As ERP.Repository.Service.Address)
			Me.address = address
			Me.LoadMap()
		End Sub

		Private Sub InitializeMapProvider()
			Me.provider = New BingRestMapProvider()
			Me.provider.BingKey = VEKey
			Me.provider.UseSession = True
			Me.provider.ImagerySet = ImagerySet.CanvasGray
			Me.map.Providers.Add(provider)

			AddHandler Me.provider.SearchCompleted, AddressOf Provider_SearchCompleted
			AddHandler Me.provider.SearchError, AddressOf Provider_SearchError
		End Sub

		Private Sub Provider_SearchError(ByVal sender As Object, ByVal e As SearchErrorEventArgs)
			If Not Me.isAddtionalSearch Then
				Me.isAddtionalSearch = True
				Me.CreateRequestAndSearchAsync(Me.address)
			Else
				MessageBox.Show("Address cannot be found!")
			End If
		End Sub

		Private Sub Provider_SearchCompleted(ByVal sender As Object, ByVal e As SearchCompletedEventArgs)
			Me.visualizationLayer.Clear()

			Dim bbox() As Double = e.Locations(0).BBox

			Dim location = e.Locations(0)
			Dim point As New Telerik.WinControls.UI.Map.PointG(location.Point.Coordinates(0), location.Point.Coordinates(1))
			Dim pin As New MapPin(point)
            pin.Size = New System.Drawing.Size(20, 40)
            pin.BackColor = System.Drawing.Color.Red
            pin.ToolTipText = location.Address.FormattedAddress
			Me.map.MapElement.Layers(0).Add(pin)
			Dim zoomPoint = New Telerik.WinControls.UI.Map.PointG(e.Locations(0).Point.Coordinates(0), e.Locations(0).Point.Coordinates(1))

			If Me.map.Visible Then
				Me.map.Zoom(9, True)
				Me.map.BringIntoView(zoomPoint)
			End If

		End Sub

		Private Sub LoadMap()
			Me.visualizationLayer = TryCast(Me.map.Layers(0), MapLayer)

			Me.CreateRequestAndSearchAsync(address)
			RemoveHandler Me.address.PropertyChanged, AddressOf OnAddressPropertyChanged
			AddHandler Me.address.PropertyChanged, AddressOf OnAddressPropertyChanged
		End Sub

		Private Sub OnMapUnloaded(ByVal sender As Object, ByVal e As RoutedEventArgs)
			RemoveHandler Me.address.PropertyChanged, AddressOf OnAddressPropertyChanged
		End Sub

		Private Sub OnAddressPropertyChanged(ByVal sender As Object, ByVal e As System.ComponentModel.PropertyChangedEventArgs)
			Me.CreateRequestAndSearchAsync(Me.address)
		End Sub

		Private Sub CreateRequestAndSearchAsync(ByVal address As Repository.Service.Address)
			Dim restRequest = New SearchRequest()

			restRequest.Culture = New System.Globalization.CultureInfo("en-US")
			restRequest.Query = Me.GenerateSearchQueryFromAddress(address)
			Me.provider.SearchAsync(restRequest)
		End Sub

		Private Function GenerateSearchQueryFromAddress(ByVal address As Repository.Service.Address) As String
				' If the first search doesn't succeed - search on for City and Country.
			Dim searchQuery As String = String.Format("{0} {1} {2}",If((Not Me.isAddtionalSearch), address.AddressLine1, String.Empty), address.City,If(address.StateProvince.CountryRegion IsNot Nothing, address.StateProvince.CountryRegion.Name, address.StateProvince.Name)).Trim()

			Return searchQuery
		End Function
	End Class
End Namespace