﻿Imports ERP.Client.Analytics
Imports ERP.Client.Properties
Imports ERP.Repository
Imports System.ComponentModel
Imports System.Text
Imports Telerik.WinControls
Imports Telerik.WinControls.Analytics
Imports Telerik.WinControls.UI

Namespace ERP.Client
	Partial Public Class MainForm
		Inherits ShapedForm
'INSTANT VB NOTE: The variable infoControl was renamed since Visual Basic does not handle variables named the same as their type well:
		Private infoControl_Renamed As InfoControl
		Private storesControl As BaseGridControl
		Private vendorsControl As BaseGridControl
		Private purchasesControl As BaseGridControl
		Private inventoriesControl As BaseGridControl
		Private billOfMaterialsControl As BaseGridControl
		Private workOrdersControl As BaseGridControl
		Private individualsControl As BaseGridControl
		Private ordersControl As BaseGridControl

		Public Sub New()
			ControlTraceMonitor.AnalyticsMonitor = New GoogleAnalyticsMonitor()

			InitializeComponent()
			Me.StartPosition = FormStartPosition.CenterScreen
			Me.radCollapsiblePanel1.CollapsiblePanelElement.HeaderElement.ShowHeaderLine = False
			AddHandler topControl1.ViewLabel.TextChanged, AddressOf ViewLabel_TextChanged
			AddHandler ThemeResolutionService.ApplicationThemeChanged, AddressOf ThemeResolutionService_ApplicationThemeChanged
			AddHandler radTreeView1.SelectedNodeChanged, AddressOf RadTreeView1_SelectedNodeChanged
			radBreadCrumb1.DefaultTreeView = radTreeView1

            Me.Icon = My.Resources.ERP
            Me.Text = "ERP Demo"

			For Each item As RadTreeNode In radTreeView1.TreeViewElement.GetNodes()
				If item.Nodes.Count > 0 Then
                    item.Image = My.Resources.folder
                End If
			Next item
            topControl1.ViewLabel.Font = New Font("Segoe UI", 16, System.Drawing.FontStyle.Regular)
            topControl1.ViewLabel.Text = ""
			radCollapsiblePanel1.EnableAnimation = False
			AddHandler radCollapsiblePanel1.Collapsed, AddressOf RadCollapsiblePanel1_Collapsed
			AddHandler radCollapsiblePanel1.Expanded, AddressOf RadCollapsiblePanel1_Expanded
		End Sub

		Private Sub ViewLabel_TextChanged(ByVal sender As Object, ByVal e As EventArgs)
			radCollapsiblePanel1.HeaderText = topControl1.ViewLabel.Text
		End Sub

		Private Sub RadCollapsiblePanel1_Expanded(ByVal sender As Object, ByVal e As EventArgs)
			tableLayoutPanel1.ColumnStyles(0).Width = 267
		End Sub

		Private Sub RadCollapsiblePanel1_Collapsed(ByVal sender As Object, ByVal e As EventArgs)
			tableLayoutPanel1.ColumnStyles(0).Width = 40
		End Sub

		Private Sub ThemeResolutionService_ApplicationThemeChanged(ByVal sender As Object, ByVal args As ThemeChangedEventArgs)
			OnThemeChanged()
		End Sub

		Protected Overrides Sub OnShown(ByVal e As EventArgs)
			MyBase.OnShown(e)
			OnThemeChanged()
			radTreeView1.Nodes(2).Nodes(1).Selected = True
			If ControlTraceMonitor.AnalyticsMonitor IsNot Nothing Then
				ControlTraceMonitor.AnalyticsMonitor.TrackAtomicFeature("ApplicationStarted." & Date.Now.ToShortDateString())
			End If
		End Sub

		Private Sub RadTreeView1_SelectedNodeChanged(ByVal sender As Object, ByVal e As Telerik.WinControls.UI.RadTreeViewEventArgs)
			If ControlTraceMonitor.AnalyticsMonitor IsNot Nothing Then
				ControlTraceMonitor.AnalyticsMonitor.TrackAtomicFeature("ViewChanged." & e.Node.Name)
			End If
			Select Case e.Node.Name
				Case "instructionsNode"
					topControl1.ViewLabel.Text = "Instructions"
					AttachInfoControl("Instructions.pdf")
				Case "documentationNode"
					topControl1.ViewLabel.Text = "Documentation"
					AttachInfoControl("Documentation.pdf")
				Case "storesNode"
					topControl1.ViewLabel.Text = "Stores"
					AttachGridControl(Of StoresControl)(storesControl)
				Case "suppliersNode"
					topControl1.ViewLabel.Text = "Vendors"
					AttachGridControl(Of VendorsControl)(vendorsControl)
				Case "purchasesNode"
					topControl1.ViewLabel.Text = "Purchase Orders"
					AttachGridControl(Of PurchasesControl)(purchasesControl)
				Case "productInventoryNode"
					topControl1.ViewLabel.Text = "Product Inventory"
					AttachGridControl(Of InventoriesControl)(inventoriesControl)
				Case "billOfMaterialsNode"
					topControl1.ViewLabel.Text = "Bill Of Materials"
					AttachGridControl(Of BillOfMaterialsControl)(billOfMaterialsControl)
				Case "workOrdersNode"
					topControl1.ViewLabel.Text = "Work Orders"
					AttachGridControl(Of WorkOrdersControl)(workOrdersControl)
				Case "individualsNode"
					topControl1.ViewLabel.Text = "Individuals"
					AttachGridControl(Of IndividualsControl)(individualsControl)
				Case "ordersNode"
					topControl1.ViewLabel.Text = "Sales Orders"
					AttachGridControl(Of OrdersControl)(ordersControl)
			End Select
		End Sub

		Public Sub AttachGridControl(Of T As {BaseGridControl, New})(ByRef ctrl As BaseGridControl)
			If ctrl Is Nothing Then
				ctrl = New T()
				ctrl.Dock = DockStyle.Fill
				ctrl.Margin = New Padding(0, 0, 7, 7)
			End If

			tableLayoutPanel1.Controls.Remove(tableLayoutPanel1.GetControlFromPosition(1, 2))
			tableLayoutPanel1.Controls.Add(ctrl, 1, 2)

		End Sub
		Public Sub AttachInfoControl(ByVal document As String)
			If infoControl_Renamed Is Nothing Then
				infoControl_Renamed = New InfoControl()
				infoControl_Renamed.Dock = DockStyle.Fill
				infoControl_Renamed.Margin = New Padding(0, 0, 7, 7)
			End If

			infoControl_Renamed.InfoPdfViewer.LoadDocument("..\..\ERP.Client\Documents\" & document)
			infoControl_Renamed.DocumentName = document

			tableLayoutPanel1.Controls.Remove(tableLayoutPanel1.GetControlFromPosition(1, 2))
			tableLayoutPanel1.Controls.Add(infoControl_Renamed, 1, 2)
		End Sub

		Protected Sub OnThemeChanged()
			Dim theme_Renamed As Theme = ThemeResolutionService.GetTheme(ThemeResolutionService.ApplicationThemeName)

			For Each styleGroup_Renamed As StyleGroup In theme_Renamed.StyleGroups
				For Each propertySettingGroup_Renamed As PropertySettingGroup In styleGroup_Renamed.PropertySettingGroups
					If propertySettingGroup_Renamed.Selector.Value = "RadFormElement" Then
						For Each propertySetting_Renamed As PropertySetting In propertySettingGroup_Renamed.PropertySettings
							If propertySetting_Renamed.Name = "BackColor" Then
                                Me.BackColor = CType(propertySetting_Renamed.Value, System.Drawing.Color)
                                Return
							End If
						Next propertySetting_Renamed
					End If
					If styleGroup_Renamed.Registrations(0).ControlType = "Telerik.WinControls.UI.RadForm" AndAlso propertySettingGroup_Renamed.Selector.Value = "Telerik.WinControls.RootRadElement" Then
						For Each propertySetting_Renamed As PropertySetting In propertySettingGroup_Renamed.PropertySettings
							If propertySetting_Renamed.Name = "BackColor" Then
                                Me.BackColor = CType(propertySetting_Renamed.Value, System.Drawing.Color)
                                Return
							End If
						Next propertySetting_Renamed
					End If

				Next propertySettingGroup_Renamed

			Next styleGroup_Renamed
		End Sub

	End Class
End Namespace
